/*
 * TismTool  Tis Veugen, 2010-2011
 * Application, Simulation, Verification
 *
 * OSAL_Thread.c
 * version 5
 */
#define _GNU_SOURCE
#include <string.h>
#include <assert.h>
#include "OSAL_Gen.h"
#include "OSAL_Thread.h"
#include <pthread.h>
#include <sched.h>
#include <unistd.h> /* usleep */

#define ASSERT_THREAD(h) assert((0<(h))&&((h)<=OSAL_NOF_THREADS+1)&&(Threads[(h)-1].bOccupied))

#ifndef PTHREAD_STACK_MIN
#define PTHREAD_STACK_MIN 0x2000
#endif

typedef void* (*pthread_start_routine_t) (void*);

static pthread_mutex_t  ThreadCs;
static OSAL_Thread      Threads[ OSAL_NOF_THREADS+1 ];

void OSAL_ThreadInitialise( void )
{
  int rv;

  memset( Threads, 0, sizeof(Threads) );
  rv = pthread_mutex_init( &ThreadCs, NULL );
  assert( rv == 0 );
}

void OSAL_ThreadTerminate( void )
{
  pthread_mutex_destroy( &ThreadCs );
}

OSAL_Handle OSAL_ThreadCreate( void (*pFct)(void*), void* pArg )
{
  OSAL_Handle         h = OSAL_HANDLE_INVALID;
  int                 i;
  int                 rv;

  pthread_mutex_lock( &ThreadCs );
  for ( i=1; i<=OSAL_NOF_THREADS; i++ )
  {
    if ( ! Threads[ i ].bOccupied )
    {
      pthread_start_routine_t sr = (pthread_start_routine_t) pFct;

      rv = pthread_create(
            &Threads[ i ].Pthread,  /* thread data */
            NULL,                   /* default attributes */
            sr,                     /* thread function name */
            pArg );                 /* argument to thread function */

      if ( rv == 0 )
      {
        Threads[ i ].bOccupied = TRUE;
        h = i + 1;
      }
      break;
      /****/
    }
  }
  pthread_mutex_unlock( &ThreadCs );

  return h;
}

OSAL_Handle OSAL_SetMainThread( void )
{
  pthread_t CurrentThread = pthread_self();

  Threads[ 0 ].bOccupied = TRUE;
  Threads[ 0 ].Pthread = CurrentThread;

  return 1;
}

OSAL_Thread* OSAL_GetCurrentThread( void )
{
  OSAL_Thread*  pThread = NULL;
  OSAL_Handle   h = OSAL_GetCurrentThreadHandle();

  if ( h != OSAL_HANDLE_INVALID )
  {
    pThread = &Threads[ h-1 ];
  }

  return pThread;
}

OSAL_Handle OSAL_GetCurrentThreadHandle( void )
{
  OSAL_Handle h = OSAL_HANDLE_INVALID;
  pthread_t   CurrentThread = pthread_self();
  int         i;

  for ( i=0; i<=OSAL_NOF_THREADS; i++ )
  {
    if ( ( Threads[ i ].bOccupied )
      && ( pthread_equal( CurrentThread, Threads[ i ].Pthread ) != 0 ) )
    {
      h = i + 1;
      break;
      /****/
    }
  }

  return h;
}

void OSAL_ThreadDelete( OSAL_Handle h )
{
  ASSERT_THREAD(h);
  pthread_mutex_lock( &ThreadCs );

  pthread_join( Threads[ h-1 ].Pthread, NULL );
  Threads[ h-1 ].bOccupied = FALSE;

  pthread_mutex_unlock( &ThreadCs );
}

void OSAL_ThreadSetAffinity( OSAL_Handle h, unsigned long Mask )
{
  cpu_set_t cpuset;
  int       j;
  int       s;

  ASSERT_THREAD(h);
  __CPU_ZERO_S( sizeof(cpu_set_t), &cpuset );
  for ( j = 0; j < sizeof(unsigned long)<<3; j++ )
  {
    if ( Mask & 1 )
    {
    	__CPU_SET_S( j, sizeof(cpu_set_t), &cpuset );
    }
    Mask >>= 1;
  }

  s = pthread_setaffinity_np( Threads[ h-1 ].Pthread, sizeof(cpu_set_t), &cpuset );
  assert( s == 0 );
}

void OSAL_ThreadSleep( int MilliSeconds )
{
  usleep( 1000 * MilliSeconds );
}

void OSAL_ThreadYield( void )
{
  int       s;

  s = sched_yield();
  assert( s == 0 );
}
