/*
 * TismTool  Tis Veugen, 2010-2011
 * Application, Simulation, Verification
 *
 * OSAL_TriggerSem.c
 * version 3
 */
#include <string.h>
#include <assert.h>
#include "OSAL_Gen.h"
#include "OSAL_TriggerSem.h"
#include <pthread.h>

#define ASSERT_TRIGGER(h) assert((0<(h))&&((h)<=OSAL_NOF_TRIGGERSEMS)&&(TriggerSems[(h)-1].bOccupied))

typedef struct
{
  OSAL_Bool       bOccupied;
  int             nTriggers;
  pthread_mutex_t cs;
  pthread_cond_t  cv;
} OSAL_TriggerSem;

static pthread_mutex_t  TriggerCs;
static OSAL_TriggerSem  TriggerSems[ OSAL_NOF_TRIGGERSEMS ];

void OSAL_TriggerSemInitialise( void )
{
  int rv;

  memset( TriggerSems, 0, sizeof(TriggerSems) );
  rv = pthread_mutex_init( &TriggerCs, NULL );
  assert( rv == 0 );
}

void OSAL_TriggerSemTerminate( void )
{
  pthread_mutex_destroy( &TriggerCs );
}

OSAL_Handle OSAL_TriggerSemCreate( void )
{
  OSAL_Handle h = OSAL_HANDLE_INVALID;
  int         i;
  int         rv;

  pthread_mutex_lock( &TriggerCs );
  for ( i=0; i<OSAL_NOF_TRIGGERSEMS; i++ )
  {
    if ( ! TriggerSems[ i ].bOccupied )
    {
      rv = pthread_mutex_init( &TriggerSems[ i ].cs, NULL );
      if ( rv == 0 )
      {
        rv = pthread_cond_init( &TriggerSems[ i ].cv, NULL );
        if ( rv == 0 )
        {
          TriggerSems[ i ].bOccupied = TRUE;
          TriggerSems[ i ].nTriggers = 0;
          h = i+1;
        }
      }
      break;
      /****/
    }
  }
  pthread_mutex_unlock( &TriggerCs );

  return h;
}

void OSAL_TriggerSemWait( OSAL_Handle h )
{
  OSAL_TriggerSem* pTs;

  ASSERT_TRIGGER(h);
  pTs = &TriggerSems[ h-1 ];

  pthread_mutex_lock( &pTs->cs );
  while ( pTs->nTriggers == 0 )
  {
    pthread_cond_wait( &pTs->cv, &pTs->cs );
  }
  pTs->nTriggers = 0;
  pthread_mutex_unlock( &pTs->cs );
}

void OSAL_TriggerSemRelease( OSAL_Handle h )
{
  OSAL_TriggerSem* pTs;

  ASSERT_TRIGGER(h);
  pTs = &TriggerSems[ h-1 ];

  pthread_mutex_lock( &pTs->cs );
  if ( pTs->nTriggers == 0 )
  {
    pTs->nTriggers++;
    pthread_cond_signal( &pTs->cv );
  }
  pthread_mutex_unlock( &pTs->cs );
}

void OSAL_TriggerSemReset( OSAL_Handle h )
{
  OSAL_TriggerSem* pTs;

  ASSERT_TRIGGER(h);
  pTs = &TriggerSems[ h-1 ];

  pthread_mutex_lock( &pTs->cs );
  pTs->nTriggers = 0;
  pthread_mutex_unlock( &pTs->cs );
}

void OSAL_TriggerSemDelete( OSAL_Handle h )
{
  OSAL_TriggerSem* pTs;

  ASSERT_TRIGGER(h);
  pTs = &TriggerSems[ h-1 ];

  pthread_mutex_lock( &TriggerCs );

  pthread_cond_destroy( &pTs->cv );
  pthread_mutex_destroy( &pTs->cs );
  pTs->bOccupied = FALSE;
  pTs->nTriggers = 0;

  pthread_mutex_unlock( &TriggerCs );
}
