/*
 * TismTool © Tis Veugen, 2010-2011
 * Simulation, Verification
 *
 * TsmCbThread.c
 * version 3
 */
#include <string.h>
#include <stdio.h>
#include "OSAL_Gen.h"
#include "OSAL_Thread.h"
#include "Tsm.h"
#include "TsmThread.h"
#include "TsmCbThread.h"
#include "TsmThreadType.h"
#include "TsmThreadState.h"
#include "TsmExtQueue.h"
#include "TsmIntQueue.h"
#include "TsmClass.h"
#include "TsmEnvelope.h"
#include "TsmTrigger.h"
#include "TsmSupervisor.h"

static void TsmCbThread_CbWorker( TsmCbThread* oTsmThread );

TsmCbThread* TsmCbThread_Ctor( void )
{
  TsmCbThread* this = (TsmCbThread*) Tsm_Malloc( sizeof(TsmCbThread) );
  TsmCbThread_Ftor( this );
  return this;
}

void TsmCbThread_Ftor( TsmCbThread* this )
{
  TsmThread_Ftor( (TsmThread*) this, TsmThreadType_CB_THREAD );
  this->Class = "TsmCbThread";

  this->Prepare = (void (*)( TsmThread*, char* )) TsmCbThread_Prepare;
  this->NativeCall = (void (*)( TsmThread*, TsmEnvelope* )) TsmCbThread_NativeCall;
  this->CbHandle = (void (*)( TsmThread*, TsmEnvelope* )) TsmCbThread_CbHandle;
}

void TsmCbThread_Prepare( TsmCbThread* this, char* sName )
{
  this->mbHalt = FALSE;
  this->mThread = OSAL_ThreadCreate( (void (*)(void *)) TsmCbThread_CbWorker, this );
  if ( OSAL_HANDLE_INVALID == this->mThread )
  {
    TsmException_Quit( "TsmCbThread_Prepare: OSAL_ThreadCreate" );
  }
  strcpy( this->mName, sName );
  this->mSequence = TsmSupervisor_Register2( (TsmThread*) this, this->mType );
  this->mExternalQueue.SetSeqId( (TsmQueue*) &this->mExternalQueue, this->mSequence );
  this->mInternalQueue.SetSeqId( (TsmQueue*) &this->mInternalQueue, this->mSequence );
}

void TsmCbThread_NativeCall( TsmCbThread* this, TsmEnvelope* oEnvelope )
{
  TsmTrigger* lTrigger = TsmEnvelope_GetTrigger( oEnvelope );

  lTrigger->SetDBU( lTrigger );
  this->mExternalQueue.Put( (TsmQueue*) &this->mExternalQueue, oEnvelope );
}

void TsmCbThread_CbHandle( TsmCbThread* this, TsmEnvelope* oEnvelope )
{
  TsmTrigger* lTrigger = TsmEnvelope_GetTrigger( oEnvelope );
  TsmClass*   lTarget  = TsmEnvelope_GetTarget( oEnvelope );

  lTarget->ExecuteNative( lTarget, lTrigger->Command );
}

static void TsmCbThread_CbWorker( TsmCbThread* oTsmThread )
{
  TsmEnvelope   lEnvelope;
  TsmTrigger*   lTrigger = NULL;
  OSAL_Thread*  oThread;
  int           rv;

  oTsmThread->TriggerWait( (TsmThread*) oTsmThread );

  oThread = OSAL_GetCurrentThread();
  rv = setjmp( oThread->JmpBuf );
  lTrigger = oTsmThread->mLocalTrigger;
  switch ( rv )
  {
  case TsmException_OK :
    break;
  case TsmException_QUIT :
    Tsm_ShowException( "Unexpected error" );
    lTrigger->DBU( lTrigger );
    break;
  case TsmException_RV :
    TsmSupervisor_SetState2( oTsmThread->mThread, TsmThreadState_INTERRUPTED );
    lTrigger->DBU( lTrigger );
    break;
  }

  while ( ! oTsmThread->mbHalt )
  {
    /* Wait for message controlled by Supervisor */
    TsmSupervisor_SetState2( oTsmThread->mThread, TsmThreadState_WAITING_MSG );

    if ( oTsmThread->mExternalQueue.Dequeue
          ( (TsmQueue*) &oTsmThread->mExternalQueue, &lEnvelope ) )
    {
      lTrigger = TsmEnvelope_GetTrigger( &lEnvelope );

      Tsm_Seq( "F" );
      oTsmThread->mLocalTrigger = lTrigger;
      oTsmThread->CbHandle( (TsmThread*) oTsmThread, &lEnvelope );
      lTrigger->DBU( lTrigger );
    }
    else
    {
      if ( oTsmThread->mInternalQueue.Dequeue
            ( (TsmQueue*) &oTsmThread->mInternalQueue, &lEnvelope ) )
      {
        lTrigger = TsmEnvelope_GetTrigger( &lEnvelope );

        Tsm_Seq( "J" );
        oTsmThread->mLocalTrigger = lTrigger;
        oTsmThread->Handle( (TsmThread*) oTsmThread, &lEnvelope );
        lTrigger->DBU( lTrigger );
      }
    }
  }
}
