/*
 * TismTool © Tis Veugen, 2010-2011
 * Simulation, Verification
 *
 * TsmRecorder.c
 * version 1
 */
#include <stdio.h>
#include <string.h>
#include "TsmRecorder.h"
#include "Tsm.h"
#include "TsmClass.h"
#include "TsmPort.h"
#include "TsmSupervisor.h"

int TsmRecorder_NumberOfUnhandledFunctions = 0;
int TsmRecorder_NumberOfUnvisitedStates = 0;
int TsmRecorder_NumberOfUnvisitedTransitions = 0;
int TsmRecorder_NumberOfSyncReturnErrors = 0;
int TsmRecorder_NumberOfQueueOverflows = 0;

static TsmArray Records;  /* <TsmStr> */
static int      NrCalls = 0;
static TsmBool  bEnable = FALSE;
static char*    Separator = "============================================================";

void TsmRecorder_Initialise( void )
{
  TsmArray_Malloc( &Records, 8, 8 );
}

void TsmRecorder_Terminate( void )
{
  TsmArray_Free( &Records );
}

int TsmRecorder_GetNumberOfRecords( void )
{
  return Records.Number;
}

int TsmRecorder_GetNumberOfExtCalls( void )
{
  return NrCalls;
}

void TsmRecorder_Reset( void )
{
  TsmArray_FreeItems( &Records );
  NrCalls = 0;
  bEnable = FALSE;
}
void TsmRecorder_Enable( void )
{
  bEnable = TRUE;
}

void TsmRecorder_AddNative( char* sNative )
{
  static const char* sh = "NaTr: ";

  TsmStr sr = Tsm_Malloc( strlen(sh) + strlen(sNative) + 1 );
  sprintf( sr, "%s%s", sh, sNative );
  TsmArray_Add( &Records, sr );
}

void TsmRecorder_AddCall( char* sCall, TsmClass* oSource, TsmPort* oPort )
{
  TsmClass* lTarget = oPort->Target;
  TsmStr    sr;

  sr = Tsm_Malloc( TSM_NCH_RECORDER );
  sprintf( sr, "C,%s,%d,%d,", sCall, oSource->SeqId, lTarget->SeqId );
  oPort->ToSeq( oPort, sr+strlen(sr) );
  Tsm_Seq( sr );

  sprintf( sr, "Call: %s @ ", sCall );
  oSource->FullName( (TsmBase*) oSource, sr+strlen(sr) );
  strcat( sr, " -> " );
  lTarget->FullName( (TsmBase*) lTarget, sr+strlen(sr) );

  sr = Tsm_Restring( sr );
  TsmArray_Add( &Records, sr );

  if ( oSource->IsMock || lTarget->IsMock )
  {
    NrCalls++;
  }
}

void TsmRecorder_AddStates( char* sStates )
{
  static const char* sh = "States: ";

  TsmStr sr = Tsm_Malloc( strlen(sh) + strlen(sStates) + 1 );
  sprintf( sr, "%s%s", sh, sStates );
  TsmArray_Add( &Records, sr );
}

void TsmRecorder_UnhandledFunction( char* sEvent )
{
  TsmRecorder_NumberOfUnhandledFunctions++;
  TsmRecorder_Show( sEvent );
}

void TsmRecorder_UnvisitedState( char* sEvent )
{
  TsmRecorder_NumberOfUnvisitedStates++;
  Tsm_Log2( "Unvisited state: ", sEvent );
}

void TsmRecorder_UnvisitedTransition( char* sState, char* sEvent )
{
  TsmRecorder_NumberOfUnvisitedTransitions++;
  Tsm_Log( Separator );
  Tsm_Log2( "@ state: ", sState );
  Tsm_Log2( "Unvisited transition: ", sEvent );
}

void TsmRecorder_SyncReturnError( char* sEvent )
{
  TsmRecorder_NumberOfSyncReturnErrors++;
  TsmRecorder_Show( sEvent );
}

void TsmRecorder_QueueOverflow( void )
{
  TsmRecorder_NumberOfQueueOverflows++;
  TsmRecorder_Show( "**** QUEUE OVERFLOW " );
}

void TsmRecorder_Show( char* sEvent )
{
  if ( bEnable )
  {
    int   i;
    char* sr;

    Tsm_Log( Separator );
    Tsm_Log( sEvent );

    for ( i=0; i<Records.Number; i++ )
    {
      Tsm_Log( TsmArray_At( &Records, i ) );
    }

    sr = Tsm_Malloc( TSM_NCH_RECORDER );
    TsmSupervisor_SnappedClasses( sr );
    Tsm_Log( sr );
    Tsm_Free( sr );
  }
  Tsm_Seq0( "X," );
  Tsm_Seq( sEvent );
}

void TsmRecorder_Show2( char* sEvent, int iNrRecords )
{
  if ( bEnable )
  {
    int   i;
    char* sr;

    Tsm_Log( Separator );
    Tsm_Log( sEvent );

    for ( i=0; i<iNrRecords; i++ )
    {
      Tsm_Log( TsmArray_At( &Records, i ) );
    }
    Tsm_Log( "=== start of cycle ===" );
    for ( i=iNrRecords; i<Records.Number; i++ )
    {
      Tsm_Log( TsmArray_At( &Records, i ) );
    }

    sr = Tsm_Malloc( TSM_NCH_RECORDER );
    TsmSupervisor_GetCurrentSituation( sr );
    Tsm_Log( sr );
    Tsm_Free( sr );
  }
  Tsm_Seq0( "X," );
  Tsm_Seq( sEvent );
}
