/*
 * TismTool © Tis Veugen, 2010-2011
 * Simulation, Verification
 *
 * TsmSvrHelpers.c
 * version 1
 */
#include <stdio.h>
#include <string.h>
#include "TsmSvrHelpers.h"

/******************** TsmThreadData ********************/

TsmBool TsmThreadData_HasMessage( TsmThreadData* this )
{
  return this->TsmThread->HasMessage( this->TsmThread );
}

void TsmThreadData_ToState( TsmThreadData* this, TsmStr sout )
{
  sprintf( sout, "Thread %d Type %d State %d ThreadId %d",
    this->ThreadId, (int) this->Type, (int) this->State,
    this->TsmThread->GetThread( this->TsmThread ) );
}

/******************** TsmSnapshot ********************/

TsmSnapshot* TsmSnapshot_Ctor( char* sSnapClasses, char* sSnapCbThread, char* sSnapThreads )
{
  TsmSnapshot* this = (TsmSnapshot*) Tsm_Malloc( sizeof(TsmSnapshot) );
  TsmSnapshot_Ftor( this, sSnapClasses, sSnapCbThread, sSnapThreads );
  return this;
}

void TsmSnapshot_Ftor( TsmSnapshot* this, char* sSnapClasses, char* sSnapCbThread, char* sSnapThreads )
{
  this->Parent = NULL;
  this->SnapClasses  = sSnapClasses;
  this->SnapCbThread = sSnapCbThread;
  this->SnapThreads  = sSnapThreads;
}

void TsmSnapshot_Gtor( TsmSnapshot* this )
{
  Tsm_Free( this->SnapClasses );
  Tsm_Free( this->SnapCbThread );
  Tsm_Free( this->SnapThreads );
}

void TsmSnapshot_Dtor( TsmSnapshot* this )
{
  TsmSnapshot_Gtor( this );
  Tsm_Free( this );
}

void TsmSnapshot_ToString( TsmSnapshot* this, TsmStr sout )
{
  sprintf( sout, "{%s &%s &%s}",
    this->SnapClasses, this->SnapCbThread, this->SnapThreads );
}

TsmBool TsmSnapshot_Equals( TsmSnapshot* this, void* o2 )
{
  TsmSnapshot* ss2 = (TsmSnapshot*) o2;
  return ( (strcmp(this->SnapClasses,ss2->SnapClasses)==0)
        && (strcmp(this->SnapCbThread,ss2->SnapCbThread)==0)
        && (strcmp(this->SnapThreads,ss2->SnapThreads)==0) );
}

/******************** TsmScenItem ********************/

TsmScenItem* TsmScenItem_Ctor( void )
{
  TsmScenItem* this = (TsmScenItem*) Tsm_Malloc( sizeof(TsmScenItem) );
  TsmScenItem_Ftor( this );
  return this;
}

void TsmScenItem_Ftor( TsmScenItem* this )
{
  TsmObject_Ftor( (TsmObject*) this );
  this->Class = "TsmScenItem";
  this->RefCount = 1;

  this->Dtor = (Tsm_pfObj) TsmScenItem_Dtor;
}

void TsmScenItem_Dtor( TsmScenItem* this )
{
  this->RefCount--;
  if ( this->RefCount == 0 )
  {
    Tsm_Free( this );
  }
}

void TsmScenItem_AddRef( TsmScenItem* this )
{
  this->RefCount++;
}

void TsmScenItem_ToString( TsmScenItem* this, TsmStr sout )
{
  switch ( this->Kind )
  {
  case TsmScenItemKind_RELEASE_CB_MSG :
    sprintf( sout, "CB_MSG" );
    break;
  case TsmScenItemKind_RELEASE_CB_RV2 :
    sprintf( sout, "CB_RV2" );
    break;
  case TsmScenItemKind_RELEASE_MSG :
    sprintf( sout, "MSG %d", this->ThreadData->ThreadId );
    break;
  case TsmScenItemKind_RELEASE_RV2 :
    sprintf( sout, "RV2 %d", this->ThreadData->ThreadId );
    break;
  case TsmScenItemKind_NATIVE :
    sprintf( sout, "NATIVE %5d ", this->Index );
    this->Receiver->FullName( (TsmBase*) this->Receiver, sout+strlen(sout) );
    break;
  case TsmScenItemKind_CHOICE :
    sprintf( sout, "CHOICE %d", this->Index );
    break;
  case TsmScenItemKind_QO :
    sprintf( sout, "QO" );
    break;
  default :
    sprintf( sout, "TismBUG" );
    break;
  }
}

/******************** TsmScenario ********************/

static int TsmScenario_ScenarioId = 0;

void TsmScenario_NewScenario( TsmScenario* this, TsmScenario* oParent )
{
  this->Id = TsmScenario_ScenarioId++;
  this->Parent = oParent;
  TsmArray_Malloc( &this->Items, 8, 8 );
  this->Snapshot = NULL;
}

TsmScenario* TsmScenario_Ctor( void )
{
  TsmScenario* this = (TsmScenario*) Tsm_Malloc( sizeof(TsmScenario) );

  TsmScenario_ScenarioId = 0;
  TsmScenario_NewScenario( this, NULL );
  return this;
}

TsmScenario* TsmScenario_Ctor2( TsmScenario* oParent, TsmArray* oItems )
{
  TsmScenario* this = (TsmScenario*) Tsm_Malloc( sizeof(TsmScenario) );
  TsmScenario_Ftor2( this, oParent, oItems );
  return this;
}

void TsmScenario_Ftor2( TsmScenario* this, TsmScenario* oParent, TsmArray* oItems )
{
  int          i;
  TsmScenItem* si;

  this->Id = TsmScenario_ScenarioId++;
  this->Parent = oParent;
  TsmArray_Malloc( &this->Items, oItems->Number, 8 );

  for ( i=0; i<oItems->Number; i++ )
  {
    si = (TsmScenItem*) TsmArray_At( oItems, i );
    TsmScenItem_AddRef( si );
    TsmArray_Add( &this->Items, si );
  }

  this->Snapshot = oParent->Snapshot;
}

void TsmScenario_Gtor( TsmScenario* this )
{
  TsmArray_DtorItems( &this->Items );
  if ( this->Snapshot != NULL )
  {
    TsmSnapshot_Dtor( this->Snapshot );
  }
}

void TsmScenario_Dtor( TsmScenario* this )
{
  TsmScenario_Gtor( this );
  Tsm_Free( this );
}

int TsmScenario_GetParentId( TsmScenario* this )
{
  return ( this->Parent == NULL ) ? -1 : this->Parent->Id;
}

TsmScenario* TsmScenario_Extend( TsmScenario* this, TsmScenItem* oTsmScenItem )
{
  TsmScenario* lTsmScenario;
  
  lTsmScenario = TsmScenario_Ctor2( this, &this->Items );
  TsmScenario_Add( lTsmScenario, oTsmScenItem );

  return lTsmScenario;
}

TsmScenario* TsmScenario_ExtendChoice( TsmScenario* this, TsmScenItem* oTsmScenItem )
{
  TsmScenario* lTsmScenario;
  
  lTsmScenario = TsmScenario_Ctor2( this->Parent, &this->Items );
  TsmScenario_Add( lTsmScenario, oTsmScenItem );

  return lTsmScenario;
}

void TsmScenario_Add( TsmScenario* this, TsmScenItem* oTsmScenItem )
{
  TsmArray_Add( &this->Items, oTsmScenItem );
}

void TsmScenario_Update( TsmScenario* this, TsmSnapshot* oSnapshot )
{
  oSnapshot->Parent = this->Snapshot;
  this->Snapshot = oSnapshot;
}

TsmBool TsmScenario_HasCycle( TsmScenario* this )
{
  TsmSnapshot* lSnapshot = this->Snapshot;

  while ( NULL != (lSnapshot = lSnapshot->Parent) )
  {
    if ( TsmSnapshot_Equals( lSnapshot, this->Snapshot ) )
    {
      return TRUE;
    }
  }
  return FALSE;
}

void TsmScenario_ToString( TsmScenario* this, TsmStr sout )
{
  int i;
  TsmScenItem* si;

  *sout = '\0';
  for ( i=0; i<this->Items.Number; i++ )
  {
    sprintf( sout+strlen(sout), "%3d ", i );
    si = (TsmScenItem*) TsmArray_At( &this->Items, i );
    TsmScenItem_ToString( si, sout+strlen(sout) );
    strcat( sout, "\n" );
  }
}

/******************** TsmSnapdata ********************/

TsmSnapdata* TsmSnapdata_Ctor( int iNrRecords, int iNrCalls, TsmScenario* oScenario )
{
  TsmSnapdata* this = (TsmSnapdata*) Tsm_Malloc( sizeof(TsmSnapdata) );
  TsmSnapdata_Ftor( this, iNrRecords, iNrCalls, oScenario );
  return this;
}

void TsmSnapdata_Ftor( TsmSnapdata* this, int iNrRecords, int iNrCalls, TsmScenario* oScenario )
{
  this->NrRecords = iNrRecords;
  this->NrCalls   = iNrCalls;
  this->Scenario  = oScenario;
}
